package top.daxian1218.dust.license;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import org.apache.commons.lang3.StringUtils;
import top.daxian1218.dust.commons.log.LoggerHelper;
import top.daxian1218.dust.commons.util.Base64Util;
import top.daxian1218.dust.license.device.DeviceCollector;
import top.daxian1218.dust.license.util.LicenseFileUtil;
import top.daxian1218.dust.license.util.RSAUtil;

import java.io.*;

/**
 * @author daxian
 */
public class LicenseService {

    private final ILicenseVerifier licenseVerifier;

    public LicenseService(ILicenseVerifier licenseVerifier) {
        if (licenseVerifier == null) {
            throw new LicenseException("授权逻辑未实现", null);
        }

        this.licenseVerifier = licenseVerifier;
    }

    /**
     * 读取签名文件，获取发行公钥
     */
    public void readSignInfo() {
        File file = new File(LicenseConstant.SIGN_FILE_NAME);
        if (!file.exists()) {
            throw new LicenseException("文件签名文件不存在, 请联系供应商提供签名的程序", null);
        }

        String signLicense;
        try {
            signLicense = LicenseFileUtil.readAll(file);
        } catch (IOException e) {
            throw new LicenseException("签名文件读取失败", null);
        }

        JSONObject jo = JSONObject.parseObject(Base64Util.decodeEx(signLicense));
        if (jo == null || jo.isEmpty()) {
            throw new LicenseException("授权失败，无效程序证书", null);
        }

        LicenseConfig.INSTANCE.setCpuCheck(jo.getBooleanValue("cpuCheck"));
        LicenseConfig.INSTANCE.setMainboardCheck(jo.getBooleanValue("mainboardCheck"));
        LicenseConfig.INSTANCE.setHostNameCheck(jo.getBooleanValue("hostCheck"));
        LicenseConfig.INSTANCE.setMacCheck(jo.getBooleanValue("macCheck"));

        if (!LicenseConfig.INSTANCE.isMacCheck()
                && !LicenseConfig.INSTANCE.isCpuCheck()
                && !LicenseConfig.INSTANCE.isHostNameCheck()
                && !LicenseConfig.INSTANCE.isMainboardCheck()) {
            throw new LicenseException("授权失败，无效程序证书内容", null);
        }

        String sign = jo.getString("sign");
        String fileName = jo.getString("name");
        if (StringUtils.isEmpty(fileName) || StringUtils.isEmpty(sign)) {
            throw new LicenseException("授权失败，签名文件信息不存在", null);
        }

        File signFile = new File(fileName);
        if (!signFile.exists()) {
            throw new LicenseException("授权失败, 被签名文件不存在", null);
        }

        String signFileHash = null;
        String publicKey = jo.getString("publicKey");
        try {
            signFileHash = LicenseFileUtil.hash(new FileInputStream(signFile));
        } catch (FileNotFoundException e) {
            // 上面已经判断过
            LoggerHelper.error(e);
        }


        if (!RSAUtil.checkSign(signFileHash, sign, publicKey)) {
            throw new LicenseException("授权失败，签名文件已修改", null);
        }

        LicenseConfig.INSTANCE.setHash(signFileHash);
        LicenseConfig.INSTANCE.setSignedFile(fileName);
        LicenseConfig.INSTANCE.setPublicKey(jo.getString("publicKey"));
    }

    /**
     * 使用公钥加密设备指纹生成种子文件
     */
    public void generateTorrentFile() {
        if (StringUtils.isEmpty(LicenseConfig.INSTANCE.getPublicKey())) {
            throw new LicenseException("设备公钥缺失", null);
        }

        DeviceCollector collector = new DeviceCollector();

        File file = new File(LicenseConstant.TORRENT_FILE);
        try {
            if (!file.exists()) {

                if (file.isDirectory()) {
                    LicenseFileUtil.deleteDirectory(file);
                }
                file.createNewFile();
                LicenseFileUtil.writeAll(file, LicenseConfig.INSTANCE.getHash() + "."
                        + Base64Util.encodeEx(collector.getDeviceFingerPrintOfRSA()));
            }
        } catch (IOException e) {
            throw new LicenseException("License种子文件无法生成", e);
        }
    }

    /**
     * 加载设备指纹到内存
     */
    public String getDeviceFingerPrint() {
        DeviceCollector collector = new DeviceCollector();
        return collector.getDeviceFingerPrint();
    }

    /**
     * 获取License证书信息
     * @return License信息
     */
    public LicenseInfo getLicense() {
        this.readSignInfo();

        LicenseConfig.INSTANCE.setDeviceFingerPrint(getDeviceFingerPrint());

        File file = new File(LicenseConstant.LICENSE_FILE);

        if (!file.exists()) {
            throw new LicenseException("License授权证书不存在, 请上传license.tor文件, 从供应商处获取授权证书", null);
        }

        try {
            String licenseData = Base64Util.decodeEx(LicenseFileUtil.readAll(file));
            int sepIndex = licenseData.indexOf(".");
            if (sepIndex <= 0) {
                throw new LicenseException("License授权证书无效格式", null);
            }

            LoggerHelper.info("证书内容:" + licenseData);
            String license = licenseData.substring(0, sepIndex);
            String sign = licenseData.substring(sepIndex + 1);

            LoggerHelper.info("公钥:" + LicenseConstant.SIGN_PUBLIC_KEY);
            LoggerHelper.info("签名:" + sign);
            LoggerHelper.info("证书:" + license);
            boolean isSigned = RSAUtil.checkSign(license, sign, LicenseConstant.SIGN_PUBLIC_KEY);
            if (!isSigned) {
                throw new LicenseException("License授权证书签名错误", null);
            }

            LicenseInfo info = JSON.parseObject(license, LicenseInfo.class);
            if (!StringUtils.equals(info.getDeviceFingerPrint(), LicenseConfig.INSTANCE.getDeviceFingerPrint())) {
                throw new LicenseException("License授权证书设备不匹配", null);
            }

            LicenseConfig.INSTANCE.setLicense(info);
            return info;
        } catch (IOException e) {
            throw new LicenseException("License授权证书读取失败", e);
        }
    }

    /**
     * 校验License证书
     */
    public void verifyLicense() throws LicenseException {
        this.readSignInfo();

        LicenseConfig.INSTANCE.setDeviceFingerPrint(getDeviceFingerPrint());

        File file = new File(LicenseConstant.LICENSE_FILE);

        if (!file.exists()) {
            throw new LicenseException("License授权证书不存在, 请上传license.tor文件, 从供应商处获取授权证书", null);
        }

        LicenseInfo info = getLicense();

        if (!this.licenseVerifier.Verify(info)) {
            LicenseConfig.INSTANCE.setLicense(null);
            throw new LicenseException(this.licenseVerifier.GetLastError(), null);
        }

        LicenseConfig.INSTANCE.setLicense(info);
    }
}
