package top.daxian1218.dust.license.util;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import top.daxian1218.dust.commons.exception.DustSystemException;
import top.daxian1218.dust.commons.util.Base64Util;
import top.daxian1218.dust.license.LicenseException;

import javax.crypto.BadPaddingException;
import javax.crypto.Cipher;
import javax.crypto.IllegalBlockSizeException;
import javax.crypto.NoSuchPaddingException;
import java.math.BigInteger;
import java.nio.charset.StandardCharsets;
import java.security.*;
import java.security.interfaces.RSAPublicKey;
import java.security.spec.InvalidKeySpecException;
import java.security.spec.PKCS8EncodedKeySpec;
import java.security.spec.X509EncodedKeySpec;
import java.util.HashMap;
import java.util.Map;

public class RSAUtil {
    static Logger logger = LoggerFactory.getLogger(RSAUtil.class);
    /** 指定key的大小 */
    private static int KEY_SIZE = 2048;

    public static final String CHAR_ENCODING = "UTF-8";
    public static final String AES_ALGORITHM = "AES/CBC/PKCS5Padding";
    public static final String RSA_ALGORITHM = "RSA/ECB/PKCS1Padding";
    public static final String ALGORITHM = "RSA";
    public static final String SIGN_ALGORITHM = "SHA256WithRSA";

    /**
     * 生成密钥对
     * @exception DustSystemException
     */
    public static Map<String, String> generateKeyPair() {
        /** RSA算法要求有一个可信任的随机数源 */
        SecureRandom sr = new SecureRandom();
        /** 为RSA算法创建一个KeyPairGenerator对象 */
        KeyPairGenerator kpg = null;
        try {
            kpg = KeyPairGenerator.getInstance(ALGORITHM);
        } catch (NoSuchAlgorithmException e) {
            throw new DustSystemException("algorithm error", "1001", e);
        }
        /** 利用上面的随机数据源初始化这个KeyPairGenerator对象 */
        kpg.initialize(KEY_SIZE, sr);
        /** 生成密匙对 */
        KeyPair kp = kpg.generateKeyPair();
        /** 得到公钥 */
        Key publicKey = kp.getPublic();

        /** 得到私钥 */
        Key privateKey = kp.getPrivate();
        // 得到模
        RSAPublicKey rsaPublicKey = (RSAPublicKey) publicKey;
        BigInteger modulus = rsaPublicKey.getModulus();

        Map<String, String> map = new HashMap<String, String>();
        map.put("publicKey", Base64Util.encodeOfBytes(publicKey.getEncoded()));
        map.put("privateKey", Base64Util.encodeOfBytes(privateKey.getEncoded()));
        map.put("modulus", Base64Util.encodeOfBytes(modulus.toByteArray()));
        return map;
    }

    /**
     * 加密方法 source： 源数据
     */
    public static String encrypt(String source, String publicKey) {
        Key key = null;
        try {
            key = getPublicKey(publicKey);
        } catch (NoSuchAlgorithmException e) {
            throw new DustSystemException("public key:algorithm error", "1001", e);
        } catch (InvalidKeySpecException e) {
            throw new DustSystemException("public key:key error", "1001", e);
        }
        /** 得到Cipher对象来实现对源数据的RSA加密 */
        Cipher cipher = null;
        try {
            cipher = Cipher.getInstance(RSA_ALGORITHM);
        } catch (NoSuchAlgorithmException e) {
            throw new DustSystemException("RSA: algorithm error", "1001", e);
        } catch (NoSuchPaddingException e) {
            throw new DustSystemException("RSA: padding error", "1001", e);
        }
        try {
            cipher.init(Cipher.ENCRYPT_MODE, key);
        } catch (InvalidKeyException e) {
            throw new DustSystemException("init: key error", "1001", e);
        }
        /** 执行加密操作 */
        byte[] bsEncrypt;
        try {
            bsEncrypt = cipher.doFinal(source.getBytes(StandardCharsets.UTF_8));
        } catch (IllegalBlockSizeException | BadPaddingException e) {
            throw new DustSystemException("final: key error", "1001", e);
        }
        return Base64Util.encodeOfBytes(bsEncrypt);
    }

    /**
     * 解密算法 cryptoGraph:密文
     * @exception LicenseException
     */
    public static String decrypt(String cryptoGraph, String privateKey) {
        Key key = null;
        try {
            key = getPrivateKey(privateKey);
        } catch (NoSuchAlgorithmException e) {
            throw new LicenseException("private key:algorithm error", e);
        } catch (InvalidKeySpecException e) {
            throw new LicenseException("private key:key error", e);
        }
        /** 得到Cipher对象对已用公钥加密的数据进行RSA解密 */
        Cipher cipher = null;
        try {
            cipher = Cipher.getInstance(RSA_ALGORITHM);
        } catch (NoSuchAlgorithmException e) {
            throw new LicenseException("cipher:algorithm error", e);
        } catch (NoSuchPaddingException e) {
            throw new LicenseException("cipher:algorithm error", e);
        }
        try {
            cipher.init(Cipher.DECRYPT_MODE, key);
        } catch (InvalidKeyException e) {
            throw new LicenseException("cipher init:key error", e);
        }
        byte[] b1 = Base64Util.decodeToBytes(cryptoGraph);
        /** 执行解密操作 */
        byte[] b;
        try {
            b = cipher.doFinal(b1);
        } catch (IllegalBlockSizeException e) {
            throw new LicenseException("cipher final:graph error", e);
        } catch (BadPaddingException e) {
            throw new LicenseException("cipher final:padding error", e);
        }
        return new String(b);
    }

    /**
     * 得到公钥
     *
     * @param key
     *            密钥字符串（经过base64编码）
     * @throws Exception
     */
    public static PublicKey getPublicKey(String key) throws NoSuchAlgorithmException, InvalidKeySpecException {
        X509EncodedKeySpec keySpec = new X509EncodedKeySpec(
                Base64Util.decodeToBytes(key));
        KeyFactory keyFactory = KeyFactory.getInstance(ALGORITHM);
        PublicKey publicKey = keyFactory.generatePublic(keySpec);
        return publicKey;
    }

    /**
     * 得到私钥
     *
     * @param key
     *            密钥字符串（经过base64编码）
     * @throws Exception
     */
    public static PrivateKey getPrivateKey(String key) throws NoSuchAlgorithmException, InvalidKeySpecException {
        PKCS8EncodedKeySpec keySpec = new PKCS8EncodedKeySpec(
                Base64Util.decodeToBytes(key));
        KeyFactory keyFactory = KeyFactory.getInstance(ALGORITHM);
        PrivateKey privateKey = keyFactory.generatePrivate(keySpec);
        return privateKey;
    }

    public static String sign(String content, String privateKey) {
        String charset = CHAR_ENCODING;
        try {
            PKCS8EncodedKeySpec priPKCS8 = new PKCS8EncodedKeySpec(
                    Base64Util.decodeToBytes(privateKey));
            KeyFactory keyFactory = KeyFactory.getInstance(ALGORITHM);
            PrivateKey priKey = keyFactory.generatePrivate(priPKCS8);

            Signature signature = Signature.getInstance(SIGN_ALGORITHM);

            signature.initSign(priKey);
            signature.update(content.getBytes(charset));

            byte[] signed = signature.sign();

            return Base64Util.encodeOfBytes(signed);
        } catch (Exception e) {

        }

        return null;
    }

    public static boolean checkSign(String content, String sign, String publicKey)
    {
        try
        {
            KeyFactory keyFactory = KeyFactory.getInstance(ALGORITHM);
            byte[] encodedKey = Base64Util.decodeToBytes(publicKey);
            PublicKey pubKey = keyFactory.generatePublic(new X509EncodedKeySpec(encodedKey));


            java.security.Signature signature = java.security.Signature
                    .getInstance("SHA256WithRSA");

            signature.initVerify(pubKey);
            signature.update( content.getBytes(CHAR_ENCODING) );

            return signature.verify( Base64Util.decodeToBytes(sign) );

        }
        catch (Exception e)
        {
            logger.error(e.getMessage(), e);
        }

        return false;
    }
}
